@@ -1,4 +1,4 @@
-# $Id: Embed.pm,v 1.17 1996/07/02 13:48:17 dougm Exp $
+# $Id: Embed.pm,v 1.18 1996/07/02 13:48:17 dougm Exp $
 require 5.002;
 
 package ExtUtils::Embed;
@@ -17,7 +17,7 @@ use vars qw(@ISA @EXPORT $VERSION
 	    );
 use strict;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/);
 #for the namespace change
 $Devel::embed::VERSION = "99.99";
 
@@ -222,7 +222,7 @@ sub ccdlflags {
 }
 
 sub perl_inc {
-   print " -I $Config{archlib}/CORE ";
+   print " -I$Config{archlib}/CORE ";
 }
 
 sub ccopts {
@@ -266,19 +266,19 @@ functions while building your application.
 
 ExtUtils::Embed exports the following functions:
  
-L<xsinit()>, L<ldopts()>, L<ccopts()>, L<perl_inc()>, L<ccflags()>, 
-L<ccdlflags()>, L<xsi_header()>, L<xsi_protos()>, L<xsi_body()>
+xsinit(), ldopts(), ccopts(), perl_inc(), ccflags(), 
+ccdlflags(), xsi_header(), xsi_protos(), xsi_body()
 
 =head1 FUNCTIONS
 
 =item xsinit()
 
-Generate C/C++ code for the XS intializer function.
+Generate C/C++ code for the XS initializer function.
 
 When invoked as C<`perl -MExtUtils::Embed -e xsinit --`>
 the following options are recognized:
 
-B<-o> <output filename> (Defaults to B<perlxsi.c>)
+B<-o> E<lt>output filenameE<gt> (Defaults to B<perlxsi.c>)
 
 B<-o STDOUT> will print to STDOUT.
 
@@ -340,7 +340,7 @@ B<-std>
 Output arguments for linking the Perl library and any extensions linked
 with the current Perl.
 
-B<-I> <path1:path2>
+B<-I> E<lt>path1:path2E<gt>
 
 Search path for ModuleName.a archives.  
 Default path is B<@INC>.
@@ -355,7 +355,7 @@ we should find B<auto/DBD/Oracle/Oracle.a>
 Keep in mind, you can always supply B</my/own/path/ModuleName.a>
 as an additional linker argument.
 
-B<-->  <list of linker args>
+B<-->  E<lt>list of linker argsE<gt>
 
 Additional linker arguments to be considered.
 
@@ -419,11 +419,11 @@ conflict, the additional arguments will be part of the output.
 
 For including perl header files this function simply prints:
 
- -I $Config{archlib}/CORE  
+ -I$Config{archlib}/CORE  
 
 So, rather than having to say:
 
- perl -MConfig -e 'print "-I $Config{archlib}/CORE"'
+ perl -MConfig -e 'print "-I$Config{archlib}/CORE"'
 
 Just say:
 
@@ -456,18 +456,18 @@ B<xsinit()> uses the xsi_* functions to generate most of it's code.
 =head1 EXAMPLES
 
 For examples on how to use B<ExtUtils::Embed> for building C/C++ applications
-with embedded perl, see the eg/ directory and the I<perlembed> man page.
+with embedded perl, see the eg/ directory and L<perlembed>.
  
 =head1 SEE ALSO
 
-the I<perlembed> man page
+L<perlembed>
 
 =head1 AUTHOR
 
-Doug MacEachern <dougm@osf.org>
+Doug MacEachern E<lt>F<dougm@osf.org>E<gt>
 
-Based on ideas from Tim Bunce <Tim.Bunce@ig.co.uk> and
-B<minimod.pl> by Andreas Koenig <k@anna.in-berlin.de> and Tim Bunce.
+Based on ideas from Tim Bunce E<lt>F<Tim.Bunce@ig.co.uk>E<gt> and
+B<minimod.pl> by Andreas Koenig E<lt>F<k@anna.in-berlin.de>E<gt> and Tim Bunce.
 
 =cut
 
