@@ -1160,6 +1160,16 @@ MAGIC* mg;
     return 0;
 }
 
+int
+magic_setfm(sv,mg)
+SV* sv;
+MAGIC* mg;
+{
+    sv_unmagic(sv, 'f');
+    SvCOMPILED_off(sv);
+    return 0;
+}
+
 int
 magic_setuvar(sv,mg)
 SV* sv;
